; Picture-Frame Rel. 0.3 Created by graechan 
; Comments directed to http://gimpchat.com or http://gimpscripts.com
;
; License: GPLv3
;    This program is free software: you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation, either version 3 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
;    GNU General Public License for more details.
;
;    To view a copy of the GNU General Public License
;    visit: http://www.gnu.org/licenses/gpl.html
;
;    created from a tutorial by Mahvin at http://www.mahvin.com/ 
; ------------
;| Change Log |
; ------------ 
; Rel 0.01 - Initial Release Creates a Moulded Picture Frame with a choice of Gradients and Tint Colors
; Rel 0.02 - bugfix to working with small images and made the final size = image size + frame
; Rel 0.03 - bugfix to the highlights function of script 
;;;;
(define (script-fu-picture-frame image drawable
                               f-size
							   shape
							   use-tint
							   tint
							   tint-opacity
							   use-gradient
							   gradient
							   pattern
							   highlight
							   ds-color
							   ds-off
							   ds-blur
							   hl-color
							   hl-off
							   conserve)							  

 (let* (
            (image-layer (car (gimp-image-get-active-layer image)))
			(type (car (gimp-drawable-type-with-alpha drawable)))
			(old-width (car (gimp-image-width image)))
			(old-height (car (gimp-image-height image)))
			(width old-width)
			(height old-height)
			(map-layer 0)
			(border-layer 0)
			(frame-layer 0)
			(tint-layer 0)
			(bump-layer 0)
			(ds-opacity 100)
        )	
	
	(gimp-context-push)
    (gimp-image-undo-group-start image)
	(gimp-context-set-foreground '(0 0 0))
	(gimp-context-set-background '(255 255 255))
	
;;;;scale image	
 (if (and (> old-width old-height) (> old-width 640))
	(begin
	(set! width 640)
	(set! height (/ old-height (/ old-width width)))
	)
 )
 (if (and (> old-height old-width) (> old-height 480))
	(begin
	(set! height 480)
	(set! width (/ old-width (/ old-height height)))
	)
 )
    (gimp-image-scale-full image width height 2)

;;;;increase canvas for frame	
    (gimp-image-resize image (+ width (* f-size 2)) (+ height (* f-size 2)) f-size f-size)
	(set! width (car (gimp-image-width image)))
	(set! height (car (gimp-image-height image)))

;;;;create map-layer
    (set! map-layer (car (gimp-layer-new image width height type "Map" 100 NORMAL-MODE)))
    (gimp-image-add-layer image map-layer -1)
	
;;;;create selection and blend
    (gimp-rect-select image 0 (/ height 2) f-size f-size 2 FALSE 0)
	(gimp-context-set-foreground '(0 0 0))
	(gimp-context-set-background '(255 255 255))
    (gimp-edit-blend map-layer FG-BG-RGB-MODE NORMAL-MODE GRADIENT-LINEAR 100 0 REPEAT-NONE FALSE FALSE 3 0.2 TRUE 0 (/ height 2) f-size (/ height 2))	

;;;;run curves
    (if (= shape 0) (gimp-curves-spline map-layer 0 18 #(0 170 33 100 89 159 135 63 168 184 199 96 224 164 242 104 255 104)))
	(if (= shape 1) (gimp-curves-spline map-layer 0 16 #(0 130 32 61 42 127 112 71 150 196 193 64 241 198 255 255)))
	(plug-in-autocrop-layer 1 image map-layer)
	(gimp-selection-none image)
	
;;;;create the border layer
    (script-fu-addborder image map-layer 1 1 '(0 0 0) 25)
	(set! frame-layer (car (gimp-image-get-active-layer image)))
	(gimp-drawable-set-name frame-layer "Picture Frame")
	(gimp-selection-layer-alpha frame-layer)

;;;;create the picture frame
    (python-fu-wrap-map-bucket 1 image frame-layer map-layer FALSE)	
	(gimp-selection-none image)
	(gimp-displays-flush)
	
;;;;scale the image back to original size	
	(gimp-image-scale-full image (+ old-width (* f-size 2)) (+ old-height (* f-size 2)) 2)
	(set! width (car (gimp-image-width image)))
	(set! height (car (gimp-image-height image)))
	
;;;;use gradient
	(gimp-context-set-gradient gradient)
	(if (= use-gradient TRUE) (plug-in-gradmap 1 image frame-layer))
	(gimp-displays-flush)
	
;;;;create the color layer
    (if (= use-tint TRUE)
	(begin
	(set! tint-layer (car (gimp-layer-new image width height type "Color Tint" tint-opacity SOFTLIGHT-MODE)))
    (gimp-image-add-layer image tint-layer -1)
	(gimp-context-set-foreground tint)
	(gimp-selection-layer-alpha frame-layer)
	(gimp-edit-fill tint-layer FOREGROUND-FILL)
	(gimp-selection-none image)
	(gimp-drawable-set-name frame-layer "Picture Frame")
	)
	)

;;;;create map layer
    (set! bump-layer (car (gimp-layer-new image width height type "Grain" 100 SOFTLIGHT-MODE)))
    (gimp-image-add-layer image bump-layer -1)
	(gimp-selection-layer-alpha frame-layer)
	(gimp-context-set-pattern pattern)
	(gimp-edit-fill bump-layer PATTERN-FILL)
	(gimp-selection-none image)
    (plug-in-bump-map 1 image frame-layer bump-layer 135 45 3 0 0 0 0 TRUE FALSE LINEAR)
	
;;;;highlight the picture frame
    (if (= highlight TRUE)
	(begin
	(gimp-selection-all image)
	(gimp-drawable-set-name frame-layer "Framed Picture")
	(the-picture-frame-highlights image frame-layer ds-color ds-opacity ds-off ds-off ds-blur hl-color hl-off hl-off FALSE)
	)
	)
    
;;;;flush the display?
	(gimp-displays-flush)
	
;;;;finish up by merging remaining layers
(if (= conserve FALSE)	
    (begin
	(set! frame-layer (car (gimp-image-merge-visible-layers image EXPAND-AS-NECESSARY)))
	(gimp-drawable-set-name frame-layer "Framed Picture")))
        
	(gimp-displays-flush)
	(gimp-image-undo-group-end image)
	(gimp-context-pop)

 )
)

(script-fu-register "script-fu-picture-frame"        		    
                    _"_Picture-Frame"
                     _"Creates a Moulded Picture Frame with a choice of Gradients and Tint Colors"
                    "Graechan"
                    "Graechan - http://gimpchat.com"
                    "Oct 2012"
                    "RGB*"
                    SF-IMAGE      "image"      0
                    SF-DRAWABLE   "drawable"   0
					SF-ADJUSTMENT _"Frame Size"     '(50 0 100 1 10 0 0)
					SF-OPTION     _"Frame Shape" '("Moulded 1" "Moulded 2" "Smooth")
					SF-TOGGLE     _"Use Color to Tint"   TRUE
					SF-COLOR      _"Tint Color"         '(0 0 255)
					SF-ADJUSTMENT _"Tint Opacity"     '(100 0 100 1 10 0 0)
					SF-TOGGLE     _"Use Gradient to Blend"   TRUE
					SF-GRADIENT   _"Gradient Blend" "Yellow Contrast"
					SF-PATTERN    _"Grain Pattern"  "Crack"
                    SF-TOGGLE     _"Frame Highlight"   TRUE
					SF-COLOR      _"Drop shadow color"       "black"
					SF-ADJUSTMENT _"Drop shadow Offset"    '(8 -100 100 1 10 0 1)
					SF-ADJUSTMENT _"Drop shadow blur radius" '(15 0 255 1 10 0 1)
					SF-COLOR      _"Highlight color"    '(255 255 0)
					SF-ADJUSTMENT _"Highlight Offset"      '(-2 -100 100 1 10 0 1)
					SF-TOGGLE     _"Keep the Layers"   FALSE
)

(script-fu-menu-register "script-fu-picture-frame" "<Image>/Script-Fu/Edges/")


(define (the-picture-frame-highlights image drawable
                               ds-color
							   ds-opacity
							   ds-offset-x
							   ds-offset-y
							   ds-blur
							   hl-color
							   hl-offset-x
							   hl-offset-y
							   keep-selection-in)
							  

 (let* (
            (image-layer (car (gimp-image-get-active-layer image)))
			(type (car (gimp-drawable-type image-layer))) 
			(typeA (car (gimp-drawable-type-with-alpha drawable)))
			(width (car (gimp-image-width image)))
			(height (car (gimp-image-height image)))
			(image-channel 0)
			(shadow-layer 0)
			(hl-layer 0)
			(keep-selection keep-selection-in)			
			
        )	

	(gimp-context-push)
    (gimp-image-undo-group-start image)
	(gimp-context-set-foreground '(0 0 0))
	(gimp-context-set-background '(255 255 255))
	(gimp-drawable-set-name image-layer "Image")	

;;;;check that a selection was made if not make one	
	(if (= (car (gimp-selection-is-empty image)) TRUE) (set! keep-selection FALSE))
	(if (= (car (gimp-selection-is-empty image)) TRUE) (gimp-selection-layer-alpha image-layer))	
	
;;;;save the selection    
	(gimp-selection-save image)
	(set! image-channel (car (gimp-image-get-active-drawable image)))	
	(gimp-channel-set-opacity image-channel 100)	
	(gimp-drawable-set-name image-channel "image-channel")
	(gimp-image-set-active-layer image image-layer)
	
;;;;create the shadow-layer
    (set! width (car (gimp-image-width image)))
	(set! height (car (gimp-image-height image)))
	(set! shadow-layer (car (gimp-layer-new image width height typeA "Shadow" ds-opacity NORMAL-MODE)))                       
    (gimp-image-add-layer image shadow-layer -1)
	(if (and (> ds-offset-x 0) (> ds-offset-y 0)) (gimp-image-resize image (+ ds-blur (+ width ds-offset-x)) (+ ds-blur (+ height ds-offset-y)) 0 0))
	(if (and (< ds-offset-x 0) (< ds-offset-y 0)) (gimp-image-resize image (+ (- width ds-offset-x) ds-blur) (+ (- height ds-offset-y) ds-blur) (+ (- 0 ds-offset-x) ds-blur) (+ (- 0 ds-offset-y) ds-blur)))
	(if (and (> ds-offset-x 0) (< ds-offset-y 0)) (gimp-image-resize image (+ ds-blur (+ width ds-offset-x)) (+ (- height ds-offset-y) ds-blur) 0 (+ (- 0 ds-offset-y) ds-blur)))
	(if (and (< ds-offset-x 0) (> ds-offset-y 0)) (gimp-image-resize image (+ (- width ds-offset-x) ds-blur) (+ ds-blur (+ height ds-offset-y)) (+ (- 0 ds-offset-x) ds-blur) 0))
    (gimp-layer-resize-to-image-size shadow-layer)	
    (gimp-selection-translate image ds-offset-x ds-offset-y)
    (gimp-context-set-foreground ds-color)
    (gimp-edit-fill shadow-layer FOREGROUND-FILL)
    (gimp-selection-none image)
    (plug-in-gauss-rle RUN-NONINTERACTIVE image shadow-layer ds-blur TRUE TRUE)
    (gimp-selection-load image-channel)
    (gimp-edit-clear shadow-layer)
	(plug-in-autocrop-layer 1 image shadow-layer)
	(gimp-image-resize-to-layers image)
	
;;;;create the highlight
    (set! width (car (gimp-image-width image)))
	(set! height (car (gimp-image-height image)))
	(set! hl-layer (car (gimp-layer-new image width height typeA "Highlight" 100 NORMAL-MODE)))
    (gimp-image-add-layer image hl-layer -1)
    ;(gimp-image-resize image (- width hl-offset-x) (- height hl-offset-y) (- 0 hl-offset-x) (- 0 hl-offset-y))
	(if (and (> hl-offset-x 0) (> hl-offset-y 0)) (gimp-image-resize image (+ width hl-offset-x) (+ height hl-offset-y) 0 0))
	(if (and (< hl-offset-x 0) (< hl-offset-y 0)) (gimp-image-resize image (- width hl-offset-x) (- height hl-offset-y) (- 0 hl-offset-x) (- 0 hl-offset-y)))
	(if (and (> hl-offset-x 0) (< hl-offset-y 0)) (gimp-image-resize image (+ width hl-offset-x) (- height hl-offset-y) 0 (- 0 hl-offset-y)))
	(if (and (< hl-offset-x 0) (> hl-offset-y 0)) (gimp-image-resize image (- width hl-offset-x) (+ height hl-offset-y) (- 0 hl-offset-x) 0))
    (gimp-layer-resize-to-image-size hl-layer)    
    (gimp-context-set-background hl-color)
    (gimp-selection-translate image hl-offset-x hl-offset-y)
    (gimp-edit-fill hl-layer BACKGROUND-FILL)
    (gimp-selection-none image)
    (gimp-selection-load image-channel)
	(gimp-edit-clear hl-layer)
    (plug-in-autocrop-layer 1 image hl-layer)
	(gimp-image-resize-to-layers image)	

;;;;finish up
    (gimp-image-set-active-layer image image-layer)
	(gimp-selection-load image-channel)
    (gimp-image-remove-channel image image-channel)	
(if (= keep-selection FALSE) (gimp-selection-none image))    
	(gimp-displays-flush)
	(gimp-image-undo-group-end image)
	(gimp-context-pop)

 )
)


